from .....types.chunk_quantum import ChunkQuantumType
from .chunk import ChunkRequest


class ChunkQuantumRequest(ChunkRequest):
    world_id: str | None = None
    chunk_id: str | None = None
    scope: ChunkQuantumType



import requests

from ....sdk.contracts.dtos.sdk.requests.world.delete import WorldDeleteRequest
from ....sdk.contracts.dtos.sdk.responses.response import Response
from ..abstract import AbstractCommand


class WorldDeleteCommand(AbstractCommand):
    async def execute(self, request: WorldDeleteRequest) -> bool:
        response: requests.Response = requests.delete(
            url=f"http://{self.options.tld}/world/{request.id}",
            timeout=self.options.timeout,
            params={"cascade": request.cascade},
        )
        return Response[bool].model_validate(response.json()).data



""" Tile Patch Command """

import requests

from ....sdk.contracts.dtos.sdk.requests.tile.patch import TilePatchRequest
from ....sdk.contracts.dtos.sdk.responses.response import Response
from ..abstract import AbstractCommand


class TilePatchCommand(AbstractCommand):
    async def execute(self, request: TilePatchRequest) -> bool:
        response: requests.Response = requests.patch(
            url=f"http://{self.options.tld}/world/{request.world_id}/chunk/{request.chunk_id}/tile/{request.tile_id}",
            timeout=self.options.timeout,
            json=request.model_dump()["partial"],
        )
        return Response[bool].model_validate(response.json()).data



""" Chunk Delete Command """

import requests

from ....sdk.contracts.dtos.sdk.requests.chunk.delete import ChunkDeleteRequest
from ....sdk.contracts.dtos.sdk.responses.response import Response
from ..abstract import AbstractCommand


class ChunkDeleteCommand(AbstractCommand):
    async def execute(self, request: ChunkDeleteRequest) -> bool:
        response: requests.Response = requests.delete(
            url=f"http://{self.options.tld}/world/{request.world_id}/chunk/{request.chunk_id}",
            timeout=self.options.timeout,
            params={"parent": request.parent, "cascade": request.cascade},
        )
        return Response[bool].model_validate(response.json()).data



import logging
import traceback
from functools import wraps
from typing import Callable

from fastapi import HTTPException

from .....sdk.contracts.errors.server.dao.conflict import DaoConflictError
from .....sdk.contracts.errors.server.dao.doesnotexist import DaoDoesNotExistError
from .....sdk.contracts.errors.server.dao.inconsistency import DaoInconsistencyError
from .....sdk.contracts.errors.server.dao.unknown import DaoUnknownError

logger = logging.getLogger()


def error_handler(func: Callable) -> Callable:
    @wraps(func)
    async def wrapper(*args, **kwargs):
        try:
            return await func(*args, **kwargs)
        except HTTPException as error:
            logger.error(str(error))
            raise error from error
        except DaoConflictError as error:
            logger.error(str(error))
            raise HTTPException(status_code=409, detail=str(error)) from error
        except DaoDoesNotExistError as error:
            logger.error(str(error))
            raise HTTPException(status_code=404, detail=str(error)) from error
        except DaoUnknownError as error:
            logger.error(str(error))
            raise HTTPException(status_code=400, detail=str(error)) from error
        except DaoInconsistencyError as error:
            logger.error(str(error))
            raise HTTPException(status_code=500, detail=str(error)) from error
        except Exception as error:
            traceback.print_exc()
            logger.error(str(error))
            # catch everything else
            raise HTTPException(status_code=500, detail=f"Uncaught exception: {str(error)}") from error

    return wrapper



import logging
from enum import Enum

from ......server.clients.dao import DaoClient
from .....common.utils import generate_random_float
from ....types.entity import EntityType
from ...tiles.address import Address
from ..entity import Entity

logger = logging.getLogger()


class EntityTree(Entity):
    address: Address
    daoclient: DaoClient

    class Meta(Enum):
        SEED = 0
        SAPLING = 1
        FRUIT = 2
        MATURE = 3

    entity_type: EntityType = EntityType.TREE

    max_amount: int = 16
    mutation_rate: float = 0.01

    async def quantum(self) -> None:
        """ """
        if EntityTree.Meta(self.state) == EntityTree.Meta.SEED:
            if generate_random_float() <= self.mutation_rate:
                self.state = EntityTree.Meta.SAPLING.value
                await self.daoclient.patch(address=self.address, document={"state": self.state})
        elif EntityTree.Meta(self.state) == EntityTree.Meta.SAPLING:
            if generate_random_float() <= self.mutation_rate:
                self.state = EntityTree.Meta.FRUIT.value
                await self.daoclient.patch(address=self.address, document={"state": self.state})
        elif EntityTree.Meta(self.state) == EntityTree.Meta.FRUIT:
            if generate_random_float() <= self.mutation_rate:
                self.state = EntityTree.Meta.MATURE.value
                await self.daoclient.patch(address=self.address, document={"state": self.state})
        elif EntityTree.Meta(self.state) == EntityTree.Meta.MATURE:
            if generate_random_float() <= self.mutation_rate:
                if self.amount < self.max_amount:
                    self.amount = self.amount + 1
                    await self.daoclient.patch(address=self.address, document={"amount": self.amount})
                self.state = EntityTree.Meta.SEED.value
                await self.daoclient.patch(address=self.address, document={"state": self.state})



import logging
import sys

from ....sdk.contracts.errors.server.service import ServiceError
from ...clients.dao import DaoClient, get_mongodb
from ...factories.chunk.flat import FlatChunkFactory
from ...factories.entity.entity import EntityFactory
from ...factories.world.world import WorldFactory
from ...services.state import StateService

logger = logging.getLogger()


class ContextManager:
    # Application Level Services
    state_service: StateService | None = None

    @staticmethod
    def deferred_init():
        if ContextManager.state_service is None:
            daoclient: DaoClient = DaoClient(database=get_mongodb())

            world_factory = WorldFactory(daoclient=daoclient)
            entity_factory = EntityFactory(daoclient=daoclient)
            flatchunk_factory = FlatChunkFactory(daoclient=daoclient, entity_factory=entity_factory)

            ContextManager.state_service = StateService(
                daoclient=daoclient,
                entity_factory=entity_factory,
                world_factory=world_factory,
                flatchunk_factory=flatchunk_factory,
            )
            logger.debug("[ContextManager] assigned new state service to context manager")
        else:
            logger.debug("[ContextManager] state service already assigned")


# pylint: disable=duplicate-code
try:
    ContextManager()
except ServiceError as error:
    logger.error(error)
    sys.exit(1)



import json

from pydantic import BaseModel
from pymongo import MongoClient
from pymongo.collection import Collection
from pymongo.database import Database
from pymongo.results import DeleteResult, InsertOneResult, UpdateResult

from ...sdk.common.config.environment import demand_env_var, demand_env_var_as_int
from ...sdk.common.utils import address_type, get_document_id_from_address
from ...sdk.contracts.dtos.entities.entity import Entity
from ...sdk.contracts.dtos.tiles.address import Address
from ...sdk.contracts.dtos.tiles.chunk import Chunk
from ...sdk.contracts.dtos.tiles.tile import Tile
from ...sdk.contracts.dtos.tiles.world import World
from ...sdk.contracts.errors.server.dao.doesnotexist import DaoDoesNotExistError
from ...sdk.contracts.errors.server.dao.unknown import DaoUnknownError
from ...sdk.contracts.types.dao_document import DaoDocumentType


def get_mongodb() -> Database:
    hostname: str = demand_env_var(name="DARKNESS_MONGODB_HOST")
    port: int = demand_env_var_as_int(name="DARKNESS_MONGODB_PORT")
    database: str = demand_env_var(name="DARKNESS_MONGODB_DATABASE")
    return MongoClient(hostname, port)[database]


class DaoClient(BaseModel):
    database: Database
    collections: dict[DaoDocumentType, Collection] = {}

    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)

        # our collections will match our document types
        for member in DaoDocumentType:
            # Create a collection for each document type
            self.collections[DaoDocumentType[member.name]] = self.database[member.value]

            # We add a single column index on the business logic layer `id` for our lookups:
            # db.<collection>.createIndex( { <field>: <sortOrder> } )
            self.collections[DaoDocumentType[member.name]].create_index({"id": 1})

    class Config:
        arbitrary_types_allowed = True

    # Single document delete
    async def delete(self, address: Address) -> DeleteResult:
        doc_type = address_type(address)
        document_id: str = get_document_id_from_address(address=address, doc_type=doc_type)
        return self.collections[doc_type].delete_one({"id": document_id})

    # single document get with typing!
    async def get(self, address: Address) -> World | Chunk | Tile | Entity:
        doc_type: DaoDocumentType = address_type(address=address)
        document_id: str = get_document_id_from_address(address=address, doc_type=doc_type)
        result: dict | None = self.collections[doc_type].find_one({"id": document_id})

        if result is None:
            raise DaoDoesNotExistError("no document found")

        if doc_type == DaoDocumentType.ENTITY:
            return Entity.model_validate(result)

        if doc_type == DaoDocumentType.TILE:
            return Tile.model_validate(result)

        if doc_type == DaoDocumentType.CHUNK:
            return Chunk.model_validate(result)

        if doc_type == DaoDocumentType.WORLD:
            return World.model_validate(result)

        raise DaoUnknownError("invalid document type")

    async def get_all(self, doc_type: DaoDocumentType) -> list[World] | list[Chunk] | list[Tile] | list[Entity]:
        results = self.collections[doc_type].find()
        documents: list[World] | list[Chunk] | list[Tile] | list[Entity] = []
        for result in results:
            if doc_type == DaoDocumentType.ENTITY:
                documents.append(Entity.model_validate(result))
            elif doc_type == DaoDocumentType.TILE:
                documents.append(Tile.model_validate(result))
            elif doc_type == DaoDocumentType.CHUNK:
                documents.append(Chunk.model_validate(result))
            elif doc_type == DaoDocumentType.WORLD:
                documents.append(World.model_validate(result))
            else:
                raise DaoUnknownError("invalid document type")
        return documents

    async def get_multi(
        self, addresses: list[Address], doc_type: DaoDocumentType
    ) -> list[World] | list[Chunk] | list[Tile] | list[Entity]:
        doc_ids: list = []
        for address in addresses:
            document_id: str = get_document_id_from_address(address=address, doc_type=doc_type)
            doc_ids.append(document_id)
        results = self.collections[doc_type].find({"id": {"$in": doc_ids}})
        documents: list[World] | list[Chunk] | list[Tile] | list[Entity] = []
        for result in results:
            if doc_type == DaoDocumentType.ENTITY:
                documents.append(Entity.model_validate(result))
            elif doc_type == DaoDocumentType.TILE:
                documents.append(Tile.model_validate(result))
            elif doc_type == DaoDocumentType.CHUNK:
                documents.append(Chunk.model_validate(result))
            elif doc_type == DaoDocumentType.WORLD:
                documents.append(World.model_validate(result))
            else:
                raise DaoUnknownError("invalid document type")
        return documents

    async def post(self, address: Address, document: World | Chunk | Tile | Entity) -> InsertOneResult:
        doc_type: DaoDocumentType = address_type(address=address)
        return self.collections[doc_type].insert_one(json.loads(document.model_dump_json()))

    async def patch(self, address: Address, document: dict) -> UpdateResult:
        # document CAN NOT CONTAIN set variables.  It MUST be serialized before this call, so help you
        doc_type: DaoDocumentType = address_type(address=address)
        document_id: str = get_document_id_from_address(address=address, doc_type=doc_type)
        return self.collections[doc_type].update_one(filter={"id": document_id}, update={"$set": document})



from enum import Enum
from typing import Self


class TileConnectionType(str, Enum):
    LEFT = "left"
    RIGHT = "right"
    UP = "up"
    DOWN = "down"

    # pylint: disable=no-else-return
    @classmethod
    def opposite(cls, of: Self) -> Self:
        if of == TileConnectionType.LEFT:
            return TileConnectionType.RIGHT
        elif of == TileConnectionType.RIGHT:
            return TileConnectionType.LEFT
        elif of == TileConnectionType.UP:
            return TileConnectionType.DOWN
        elif of == TileConnectionType.DOWN:
            return TileConnectionType.UP
        raise ValueError(f"Unknown tile connection type: {of}")



from pydantic import BaseModel

from .....types.tile import TileType


class ChunkCreateRequest(BaseModel):
    world_id: str | None = None

    name: str | None = None
    dimensions: tuple[int, int]
    biome: TileType



import logging
import sys

from ....client.state import StateClient
from ....sdk.contracts.errors.server.service import ServiceError

logger = logging.getLogger()


class ContextManager:
    # Application Level Services
    client: StateClient | None = None

    @staticmethod
    def deferred_init():
        if ContextManager.client is None:
            ContextManager.client = StateClient()
            logger.debug("[ContextManager] assigned new state client to context manager")
        else:
            logger.debug("[ContextManager] state client already assigned")


# pylint: disable=duplicate-code
try:
    ContextManager()
except ServiceError as error:
    logger.error(error)
    sys.exit(1)



from fastapi import APIRouter, HTTPException

from .....sdk.contracts.dtos.entities.entity import Entity
from .....sdk.contracts.dtos.sdk.requests.chunk.chunk import ChunkRequest
from .....sdk.contracts.dtos.sdk.requests.chunk.create import ChunkCreateRequest
from .....sdk.contracts.dtos.sdk.requests.chunk.delete import ChunkDeleteRequest
from .....sdk.contracts.dtos.sdk.requests.chunk.get import ChunkGetRequest
from .....sdk.contracts.dtos.sdk.requests.chunk.patch import ChunkPatchRequest
from .....sdk.contracts.dtos.sdk.requests.chunk.quantum import ChunkQuantumRequest
from .....sdk.contracts.dtos.sdk.requests.entity.delete import EntityDeleteRequest
from .....sdk.contracts.dtos.sdk.requests.entity.entity import EntityRequest
from .....sdk.contracts.dtos.sdk.requests.entity.patch import EntityPatchRequest
from .....sdk.contracts.dtos.sdk.requests.tile.delete import TileDeleteRequest
from .....sdk.contracts.dtos.sdk.requests.tile.get import TileGetRequest
from .....sdk.contracts.dtos.sdk.requests.tile.patch import TilePatchRequest
from .....sdk.contracts.dtos.sdk.requests.world.create import WorldCreateRequest
from .....sdk.contracts.dtos.sdk.requests.world.delete import WorldDeleteRequest
from .....sdk.contracts.dtos.sdk.requests.world.get import WorldGetRequest
from .....sdk.contracts.dtos.sdk.requests.world.patch import WorldPatchRequest
from .....sdk.contracts.dtos.sdk.responses.chunk.create import ChunkCreateResponse
from .....sdk.contracts.dtos.sdk.responses.chunk.get import ChunkGetResponse
from .....sdk.contracts.dtos.sdk.responses.entity.get import EntityGetResponse
from .....sdk.contracts.dtos.sdk.responses.response import Response
from .....sdk.contracts.dtos.sdk.responses.tile.get import TileGetResponse
from .....sdk.contracts.dtos.sdk.responses.world.create import WorldCreateResponse
from .....sdk.contracts.dtos.sdk.responses.world.get import WorldGetResponse
from .....sdk.contracts.dtos.tiles.chunk import Chunk
from .....sdk.contracts.dtos.tiles.tile import Tile
from .....sdk.contracts.dtos.tiles.world import World
from .....sdk.contracts.types.chunk_quantum import ChunkQuantumType
from ...common.middleware.error import error_handler
from ..context import ContextManager

# from pyinstrument import Profiler


router: APIRouter = APIRouter(
    prefix="/world",
    tags=["world"],
)


### /world


@router.get("/{world_id}")
@error_handler
async def world_get(world_id: str, full: bool = False) -> Response[WorldGetResponse]:
    request: WorldGetRequest = WorldGetRequest(id=world_id)

    if full:
        world: World = await ContextManager.state_service.world_get(request=request)
        response = Response[WorldGetResponse](data=WorldGetResponse(world=world))
    else:
        world_lite: World = await ContextManager.state_service.world_lite_get(request=request)
        response = Response[WorldGetResponse](data=WorldGetResponse(world=world_lite))

    return response


@router.delete("/{world_id}")
@error_handler
async def world_delete(world_id: str, cascade: bool) -> Response[bool]:
    request: WorldDeleteRequest = WorldDeleteRequest(id=world_id, cascade=cascade)
    success: bool = await ContextManager.state_service.world_delete(request=request)
    return Response[bool](data=success)


@router.patch("/{world_id}")
@error_handler
async def world_patch(world_id: str, partial: dict) -> Response[bool]:
    request: WorldPatchRequest = WorldPatchRequest(id=world_id, partial=partial)
    await ContextManager.state_service.world_patch(request=request)
    return Response[bool](data=True)


@router.post("")
@error_handler
async def world_create(request: WorldCreateRequest) -> Response[WorldCreateResponse]:
    world_id: str = await ContextManager.state_service.world_create(request=request)
    return Response[WorldCreateResponse](data=WorldCreateResponse(id=world_id))


### /world/world_id/chunk/


@router.post("/{world_id}/chunk")
@error_handler
async def chunk_create(world_id: str, request: ChunkCreateRequest) -> Response[ChunkCreateResponse]:
    # we ignore any passed in world_ids in this condition and over-write based on path.
    # Due to this world_id is optional within the DTO.
    request.world_id = world_id

    # with Profiler() as profiler:
    chunk_id: str = await ContextManager.state_service.chunk_create(request=request)
    # profiler.print()
    return Response[ChunkCreateResponse](data=ChunkCreateResponse(id=chunk_id))


@router.patch("/{world_id}/chunk/{chunk_id}")
@error_handler
async def chunk_patch(world_id: str, chunk_id: str, partial: dict) -> Response[bool]:
    request: ChunkPatchRequest = ChunkPatchRequest(id=world_id, chunk_id=chunk_id, partial=partial)
    await ContextManager.state_service.chunk_patch(request=request)
    return Response[bool](data=ChunkCreateResponse(id=True))


@router.get("/{world_id}/chunk/{chunk_id}")
@error_handler
async def chunk_get(world_id: str, chunk_id: str, full: bool = True) -> Response[ChunkGetResponse]:
    request: ChunkGetRequest = ChunkGetRequest(world_id=world_id, chunk_id=chunk_id)

    if full:
        chunk: Chunk = await ContextManager.state_service.chunk_get(request=request)
        response = Response[ChunkGetResponse](data=ChunkGetResponse(chunk=chunk))
    else:
        chunk: Chunk = await ContextManager.state_service.chunk_lite_get(request=request)
        response = Response[ChunkGetResponse](data=ChunkGetResponse(chunk=chunk))

    return response


@router.delete("/{world_id}/chunk/{chunk_id}")
@error_handler
async def chunk_delete(world_id: str, chunk_id: str, parent: bool, cascade: bool) -> Response[bool]:
    request: ChunkDeleteRequest = ChunkDeleteRequest(
        world_id=world_id, chunk_id=chunk_id, parent=parent, cascade=cascade
    )
    success: bool = await ContextManager.state_service.chunk_delete(request=request)
    return Response[bool](data=success)


@router.post("/{world_id}/chunk/{chunk_id}")
@error_handler
async def chunk_quantum(world_id: str, chunk_id: str, request: ChunkQuantumRequest) -> Response[bool]:
    chunk_request: ChunkRequest = ChunkRequest(world_id=world_id, chunk_id=chunk_id)
    if request.scope == ChunkQuantumType.ALL:
        await ContextManager.state_service.chunk_quantum(request=chunk_request)
    elif request.scope == ChunkQuantumType.ENTITY:
        await ContextManager.state_service.chunk_quantum_entity(request=chunk_request)
    elif request.scope == ChunkQuantumType.TILE:
        await ContextManager.state_service.chunk_quantum_tile(request=chunk_request)
    else:
        msg: str = f"unknown scope ({request.scope})"
        raise HTTPException(status_code=400, detail=msg)
    return Response[bool](data=True)


### /world/{world_id}/chunk/{chunk_id}/tile

### /world/{world_id}/chunk/{chunk_id}/tile/{tile_id}


## get
@router.get("/{world_id}/chunk/{chunk_id}/tile/{tile_id}")
@error_handler
async def tile_get(world_id: str, chunk_id: str, tile_id: str, full: bool = True) -> Response[TileGetResponse]:
    request: TileGetRequest = TileGetRequest(world_id=world_id, chunk_id=chunk_id, tile_id=tile_id)

    if full:
        tile: Tile = await ContextManager.state_service.tile_get(request=request)
        response = Response[TileGetResponse](data=TileGetResponse(tile=tile))
    else:
        tile: Tile = await ContextManager.state_service.tile_lite_get(request=request)
        response = Response[TileGetResponse](data=TileGetResponse(tile=tile))

    return response


## patch
@router.patch("/{world_id}/chunk/{chunk_id}/tile/{tile_id}")
@error_handler
async def tile_patch(world_id: str, chunk_id: str, tile_id: str, partial: dict) -> Response[bool]:
    request: TilePatchRequest = TilePatchRequest(world_id=world_id, chunk_id=chunk_id, tile_id=tile_id, partial=partial)
    await ContextManager.state_service.tile_patch(request=request)
    return Response[bool](data=True)


@router.delete("/{world_id}/chunk/{chunk_id}/tile/{tile_id}")
@error_handler
async def tile_delete(world_id: str, chunk_id: str, tile_id: str, parent: bool, cascade: bool) -> Response[bool]:
    request: TileDeleteRequest = TileDeleteRequest(
        world_id=world_id, chunk_id=chunk_id, tile_id=tile_id, parent=parent, cascade=cascade
    )
    success: bool = await ContextManager.state_service.tile_delete(request=request)
    return Response[bool](data=success)


## get
@router.get("/{world_id}/chunk/{chunk_id}/tile/{tile_id}/entity/{entity_id}")
@error_handler
async def entity_get(world_id: str, chunk_id: str, tile_id: str, entity_id: str) -> Response[EntityGetResponse]:
    request: EntityRequest = EntityRequest(world_id=world_id, chunk_id=chunk_id, tile_id=tile_id, entity_id=entity_id)
    entity: Entity = await ContextManager.state_service.entity_get(request=request)
    return Response[EntityGetResponse](data=EntityGetResponse(entity=entity))


## patch
@router.patch("/{world_id}/chunk/{chunk_id}/tile/{tile_id}/entity/{entity_id}")
@error_handler
async def entity_patch(world_id: str, chunk_id: str, tile_id: str, entity_id: str, partial: dict) -> Response[bool]:
    request: EntityPatchRequest = EntityPatchRequest(
        world_id=world_id, chunk_id=chunk_id, tile_id=tile_id, entity_id=entity_id, partial=partial
    )
    await ContextManager.state_service.entity_patch(request=request)
    return Response[bool](data=True)


### /world/{world_id}/chunk/{chunk_id}/tile/{tile_id}/entity/{entity_id}
## delete
@router.delete("/{world_id}/chunk/{chunk_id}/tile/{tile_id}/entity/{entity_id}")
@error_handler
async def entity_delete(world_id: str, chunk_id: str, tile_id: str, entity_id: str, parent: bool) -> Response[bool]:
    request: EntityDeleteRequest = EntityDeleteRequest(
        world_id=world_id, chunk_id=chunk_id, tile_id=tile_id, entity_id=entity_id, parent=parent
    )
    success: bool = await ContextManager.state_service.entity_delete(request=request)
    return Response[bool](data=success)


### /world/{world_id}/chunk/{chunk_id}/tile/{tile_id}/entity
## post (create)



import asyncio
import logging
from asyncio import Queue

from ....sdk.contracts.dtos.tiles.address import Address
from ....sdk.contracts.dtos.tiles.chunk import Chunk
from .abstract import AbstractEntityFactory

logger = logging.getLogger()


class EntityFactory(AbstractEntityFactory):
    """ """

    async def terrain_generate(self, address: Address, chunk: Chunk) -> None:
        async def step_one():
            async def consumer(queue: Queue):
                while not queue.empty():
                    local_tile_id: str = await queue.get()
                    address_tile: Address = Address.model_validate({**address.model_dump(), "tile_id": local_tile_id})
                    await self.generate(address=address_tile)
                    queue.task_done()

            queue = asyncio.Queue()
            await asyncio.gather(EntityFactory.producer(ids=chunk.ids, queue=queue), consumer(queue))

        await step_one()

    async def quantum(self, address: Address):
        chunk: Chunk = await self.daoclient.get(address=address)

        # Entity Factory Quantum
        async def step_six():
            async def consumer(queue: Queue):
                while not queue.empty():
                    local_tile_id: str = await queue.get()
                    address_tile: Address = Address.model_validate({**address.model_dump(), "tile_id": local_tile_id})
                    await self.grow_entities(address=address_tile)
                    queue.task_done()

            queue = asyncio.Queue()
            await asyncio.gather(self.producer(ids=chunk.ids, queue=queue), consumer(queue))

        await step_six()
